@@ -185,6 +185,13 @@ class Authenticator(LoggingConfigurable):
         """
     )
 
+    def get_custom_html(self, base_url):
+        """Get custom HTML for the authenticator.
+
+        .. versionadded: 1.4
+        """
+        return self.custom_html
+
     login_service = Unicode(
         help="""
         Name of the login service that this authenticator is providing using to authenticate users.
