@@ -415,3 +415,14 @@ async def test_spawner_env(db):
     for key, value in env_overrides.items():
         assert key in env
         assert env[key] == value
+
+
+async def test_hub_connect_url(db):
+    spawner = new_spawner(db, hub_connect_url="https://example.com/")
+    name = spawner.user.name
+    env = spawner.get_env()
+    assert env["JUPYTERHUB_API_URL"] == "https://example.com/api"
+    assert (
+        env["JUPYTERHUB_ACTIVITY_URL"]
+        == "https://example.com/api/users/%s/activity" % name
+    )
